@@ -79,14 +79,15 @@ ALL_CAPABILITIES = [
 ]
 
 # API version number to use when calling the service
-API_VERSION = 'v2'
+API_VERSION = 'v4'
 
 logger = getLogger(__name__)
 
 
 @unique
 class MetadataDirectiveMode(Enum):
-    """ Mode of handling metadata when copying a file """
+    """Mode of handling metadata when copying a file"""
+
     COPY = 401  #: copy metadata from the source file
     REPLACE = 402  #: ignore the source file metadata and set it to provided values
 
@@ -100,6 +101,7 @@ class LifecycleRule(TypedDict):
 
     .. _B2 Cloud Storage Lifecycle Rules: https://www.backblaze.com/docs/cloud-storage-lifecycle-rules
     """
+
     fileNamePrefix: str
     daysFromHidingToDeleting: NotRequired[PositiveInt | None]
     daysFromUploadingToHiding: NotRequired[PositiveInt | None]
@@ -117,6 +119,7 @@ class NotificationTargetConfiguration(TypedDict):
 
     `hmacSha256SigningSecret`, if present, has to be a string of 32 alphanumeric characters.
     """
+
     # TODO: add URL to the documentation
 
     targetType: Literal['webhook']
@@ -126,16 +129,26 @@ class NotificationTargetConfiguration(TypedDict):
 
 
 EVENT_TYPE = Literal[
-    'b2:ObjectCreated:*', 'b2:ObjectCreated:Upload', 'b2:ObjectCreated:MultipartUpload',
-    'b2:ObjectCreated:Copy', 'b2:ObjectCreated:Replica', 'b2:ObjectCreated:MultipartReplica',
-    'b2:ObjectDeleted:*', 'b2:ObjectDeleted:Delete', 'b2:ObjectDeleted:LifecycleRule',
-    'b2:HideMarkerCreated:*', 'b2:HideMarkerCreated:Hide', 'b2:HideMarkerCreated:LifecycleRule',]
+    'b2:ObjectCreated:*',
+    'b2:ObjectCreated:Upload',
+    'b2:ObjectCreated:MultipartUpload',
+    'b2:ObjectCreated:Copy',
+    'b2:ObjectCreated:Replica',
+    'b2:ObjectCreated:MultipartReplica',
+    'b2:ObjectDeleted:*',
+    'b2:ObjectDeleted:Delete',
+    'b2:ObjectDeleted:LifecycleRule',
+    'b2:HideMarkerCreated:*',
+    'b2:HideMarkerCreated:Hide',
+    'b2:HideMarkerCreated:LifecycleRule',
+]
 
 
 class _NotificationRule(TypedDict):
     """
     Notification Rule.
     """
+
     eventTypes: list[EVENT_TYPE]
     isEnabled: bool
     name: str
@@ -150,6 +163,7 @@ class NotificationRule(_NotificationRule):
 
     When creating or modifying a notification rule, `isSuspended` and `suspensionReason` are ignored.
     """
+
     isSuspended: NotRequired[bool]
 
 
@@ -232,8 +246,15 @@ class AbstractRawApi(metaclass=ABCMeta):
 
     @abstractmethod
     def create_key(
-        self, api_url, account_auth_token, account_id, capabilities, key_name,
-        valid_duration_seconds, bucket_id, name_prefix
+        self,
+        api_url,
+        account_auth_token,
+        account_id,
+        capabilities,
+        key_name,
+        valid_duration_seconds,
+        bucket_ids,
+        name_prefix,
     ):
         pass
 
@@ -272,8 +293,9 @@ class AbstractRawApi(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def get_file_info_by_id(self, api_url: str, account_auth_token: str,
-                            file_id: str) -> dict[str, Any]:
+    def get_file_info_by_id(
+        self, api_url: str, account_auth_token: str, file_id: str
+    ) -> dict[str, Any]:
         pass
 
     @abstractmethod
@@ -337,7 +359,7 @@ class AbstractRawApi(metaclass=ABCMeta):
         account_auth_token,
         account_id,
         max_key_count=None,
-        start_application_key_id=None
+        start_application_key_id=None,
     ):
         pass
 
@@ -428,7 +450,9 @@ class AbstractRawApi(metaclass=ABCMeta):
             headers[FILE_INFO_HEADER_PREFIX + k] = b2_url_encode(v)
         if server_side_encryption is not None:
             assert server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
             )
             server_side_encryption.add_to_upload_headers(headers)
 
@@ -482,14 +506,18 @@ class AbstractRawApi(metaclass=ABCMeta):
 
     @abstractmethod
     def set_bucket_notification_rules(
-        self, api_url: str, account_auth_token: str, bucket_id: str,
-        rules: Iterable[NotificationRule]
+        self,
+        api_url: str,
+        account_auth_token: str,
+        bucket_id: str,
+        rules: Iterable[NotificationRule],
     ) -> list[NotificationRuleResponse]:
         pass
 
     @abstractmethod
-    def get_bucket_notification_rules(self, api_url: str, account_auth_token: str,
-                                      bucket_id: str) -> list[NotificationRuleResponse]:
+    def get_bucket_notification_rules(
+        self, api_url: str, account_auth_token: str, bucket_id: str
+    ) -> list[NotificationRuleResponse]:
         pass
 
 
@@ -510,6 +538,8 @@ class B2RawHTTPApi(AbstractRawApi):
     which is relatively quick.
     """
 
+    API_VERSION = API_VERSION
+
     def __init__(self, b2_http):
         self.b2_http = b2_http
 
@@ -523,7 +553,7 @@ class B2RawHTTPApi(AbstractRawApi):
         :param args: the rest of the parameters are passed to b2
         :return: the decoded JSON response
         """
-        url = f'{base_url}/b2api/{API_VERSION}/{endpoint}'
+        url = f'{base_url}/b2api/{self.API_VERSION}/{endpoint}'
         headers = {'Authorization': auth}
         return self.b2_http.post_json_return_json(url, headers, params)
 
@@ -533,7 +563,9 @@ class B2RawHTTPApi(AbstractRawApi):
         return self.b2_http.request_content_return_json('GET', url, headers, params=params)
 
     def authorize_account(self, realm_url, application_key_id, application_key):
-        auth = f"Basic {base64.b64encode(f'{application_key_id}:{application_key}'.encode()).decode()}"
+        auth = (
+            f"Basic {base64.b64encode(f'{application_key_id}:{application_key}'.encode()).decode()}"
+        )
         return self._post_json(realm_url, 'b2_authorize_account', auth)
 
     def cancel_large_file(self, api_url, account_auth_token, file_id):
@@ -567,8 +599,9 @@ class B2RawHTTPApi(AbstractRawApi):
         if default_server_side_encryption is not None:
             if not default_server_side_encryption.mode.can_be_set_as_bucket_default():
                 raise WrongEncryptionModeForBucketDefault(default_server_side_encryption.mode)
-            kwargs['defaultServerSideEncryption'
-                  ] = default_server_side_encryption.serialize_to_json_for_request()
+            kwargs['defaultServerSideEncryption'] = (
+                default_server_side_encryption.serialize_to_json_for_request()
+            )
         if is_file_lock_enabled is not None:
             kwargs['fileLockEnabled'] = is_file_lock_enabled
         if replication is not None:
@@ -581,8 +614,15 @@ class B2RawHTTPApi(AbstractRawApi):
         )
 
     def create_key(
-        self, api_url, account_auth_token, account_id, capabilities, key_name,
-        valid_duration_seconds, bucket_id, name_prefix
+        self,
+        api_url,
+        account_auth_token,
+        account_id,
+        capabilities,
+        key_name,
+        valid_duration_seconds,
+        bucket_ids,
+        name_prefix,
     ):
         return self._post_json(
             api_url,
@@ -592,7 +632,7 @@ class B2RawHTTPApi(AbstractRawApi):
             capabilities=capabilities,
             keyName=key_name,
             validDurationInSeconds=valid_duration_seconds,
-            bucketId=bucket_id,
+            bucketIds=bucket_ids,
             namePrefix=name_prefix,
         )
 
@@ -602,7 +642,7 @@ class B2RawHTTPApi(AbstractRawApi):
             'b2_delete_bucket',
             account_auth_token,
             accountId=account_id,
-            bucketId=bucket_id
+            bucketId=bucket_id,
         )
 
     def delete_file_version(
@@ -646,7 +686,9 @@ class B2RawHTTPApi(AbstractRawApi):
 
         if encryption is not None:
             assert encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
             )
             encryption.add_to_download_headers(request_headers)
 
@@ -663,7 +705,7 @@ class B2RawHTTPApi(AbstractRawApi):
             'b2_finish_large_file',
             account_auth_token,
             fileId=file_id,
-            partSha1Array=part_sha1_array
+            partSha1Array=part_sha1_array,
         )
 
     def get_download_authorization(
@@ -675,11 +717,12 @@ class B2RawHTTPApi(AbstractRawApi):
             account_auth_token,
             bucketId=bucket_id,
             fileNamePrefix=file_name_prefix,
-            validDurationInSeconds=valid_duration_in_seconds
+            validDurationInSeconds=valid_duration_in_seconds,
         )
 
-    def get_file_info_by_id(self, api_url: str, account_auth_token: str,
-                            file_id: str) -> dict[str, Any]:
+    def get_file_info_by_id(
+        self, api_url: str, account_auth_token: str, file_id: str
+    ) -> dict[str, Any]:
         return self._post_json(api_url, 'b2_get_file_info', account_auth_token, fileId=file_id)
 
     def get_file_info_by_name(
@@ -688,11 +731,11 @@ class B2RawHTTPApi(AbstractRawApi):
         download_url = self.get_download_url_by_name(download_url, bucket_name, file_name)
         try:
             response = self.b2_http.head_content(
-                download_url, headers={"Authorization": account_auth_token}
+                download_url, headers={'Authorization': account_auth_token}
             )
             return response.headers
         except ResourceNotFound:
-            logger.debug("Resource Not Found: %s" % download_url)
+            logger.debug('Resource Not Found: %s' % download_url)
             raise FileOrBucketNotFound(bucket_name, file_name)
 
     def get_upload_url(self, api_url, account_auth_token, bucket_id):
@@ -772,7 +815,7 @@ class B2RawHTTPApi(AbstractRawApi):
         account_auth_token,
         account_id,
         max_key_count=None,
-        start_application_key_id=None
+        start_application_key_id=None,
     ):
         return self._post_json(
             api_url,
@@ -790,7 +833,7 @@ class B2RawHTTPApi(AbstractRawApi):
             account_auth_token,
             fileId=file_id,
             startPartNumber=start_part_number,
-            maxPartCount=max_part_count
+            maxPartCount=max_part_count,
         )
 
     def list_unfinished_large_files(
@@ -828,7 +871,9 @@ class B2RawHTTPApi(AbstractRawApi):
         kwargs = {}
         if server_side_encryption is not None:
             assert server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
             )
             kwargs['serverSideEncryption'] = server_side_encryption.serialize_to_json_for_request()
 
@@ -852,7 +897,7 @@ class B2RawHTTPApi(AbstractRawApi):
             fileName=file_name,
             fileInfo=file_info,
             contentType=content_type,
-            **kwargs
+            **kwargs,
         )
 
     def update_bucket(
@@ -885,8 +930,9 @@ class B2RawHTTPApi(AbstractRawApi):
         if default_server_side_encryption is not None:
             if not default_server_side_encryption.mode.can_be_set_as_bucket_default():
                 raise WrongEncryptionModeForBucketDefault(default_server_side_encryption.mode)
-            kwargs['defaultServerSideEncryption'
-                  ] = default_server_side_encryption.serialize_to_json_for_request()
+            kwargs['defaultServerSideEncryption'] = (
+                default_server_side_encryption.serialize_to_json_for_request()
+            )
         if default_retention is not None:
             kwargs['defaultRetention'] = default_retention.serialize_to_json_for_request()
         if replication is not None:
@@ -902,7 +948,7 @@ class B2RawHTTPApi(AbstractRawApi):
             account_auth_token,
             accountId=account_id,
             bucketId=bucket_id,
-            **kwargs
+            **kwargs,
         )
 
     def update_file_retention(
@@ -924,7 +970,7 @@ class B2RawHTTPApi(AbstractRawApi):
                 fileId=file_id,
                 fileName=file_name,
                 bypassGovernance=bypass_governance,
-                **kwargs
+                **kwargs,
             )
         except AccessDenied:
             raise RetentionWriteError()
@@ -961,25 +1007,23 @@ class B2RawHTTPApi(AbstractRawApi):
         encoded_name = filename.encode('utf-8')
         length_in_bytes = len(encoded_name)
         if length_in_bytes < 1:
-            raise UnusableFileName("Filename must be at least 1 character.")
+            raise UnusableFileName('Filename must be at least 1 character.')
         if length_in_bytes > 1024:
-            raise UnusableFileName("Filename is too long (can be at most 1024 bytes).")
+            raise UnusableFileName('Filename is too long (can be at most 1024 bytes).')
         lowest_unicode_value = ord(min(filename))
         if lowest_unicode_value < 32:
-            message = "Filename \"{}\" contains code {} (hex {:02x}), less than 32.".format(
-                unprintable_to_hex(filename), lowest_unicode_value, lowest_unicode_value
-            )
+            message = f'Filename "{unprintable_to_hex(filename)}" contains code {lowest_unicode_value} (hex {lowest_unicode_value:02x}), less than 32.'
             raise UnusableFileName(message)
         # No DEL for you.
         if '\x7f' in filename:
-            raise UnusableFileName("DEL character (0x7f) not allowed.")
+            raise UnusableFileName('DEL character (0x7f) not allowed.')
         if filename[0] == '/' or filename[-1] == '/':
             raise UnusableFileName("Filename may not start or end with '/'.")
         if '//' in filename:
-            raise UnusableFileName("Filename may not contain \"//\".")
+            raise UnusableFileName('Filename may not contain "//".')
         long_segment = max([len(segment.encode('utf-8')) for segment in filename.split('/')])
         if long_segment > 250:
-            raise UnusableFileName("Filename segment too long (maximum 250 bytes in utf-8).")
+            raise UnusableFileName('Filename segment too long (maximum 250 bytes in utf-8).')
 
     def upload_file(
         self,
@@ -1043,11 +1087,13 @@ class B2RawHTTPApi(AbstractRawApi):
             'Authorization': upload_auth_token,
             'Content-Length': str(content_length),
             'X-Bz-Part-Number': str(part_number),
-            'X-Bz-Content-Sha1': content_sha1
+            'X-Bz-Content-Sha1': content_sha1,
         }
         if server_side_encryption is not None:
             assert server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
             )
             server_side_encryption.add_to_upload_headers(headers)
 
@@ -1097,14 +1143,18 @@ class B2RawHTTPApi(AbstractRawApi):
             kwargs['destinationBucketId'] = destination_bucket_id
         if destination_server_side_encryption is not None:
             assert destination_server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
+            )
+            kwargs['destinationServerSideEncryption'] = (
+                destination_server_side_encryption.serialize_to_json_for_request()
             )
-            kwargs['destinationServerSideEncryption'
-                  ] = destination_server_side_encryption.serialize_to_json_for_request()
         if source_server_side_encryption is not None:
             assert source_server_side_encryption.mode == EncryptionMode.SSE_C
-            kwargs['sourceServerSideEncryption'
-                  ] = source_server_side_encryption.serialize_to_json_for_request()
+            kwargs['sourceServerSideEncryption'] = (
+                source_server_side_encryption.serialize_to_json_for_request()
+            )
 
         if legal_hold is not None:
             kwargs['legalHold'] = legal_hold.to_server()
@@ -1119,7 +1169,7 @@ class B2RawHTTPApi(AbstractRawApi):
                 account_auth_token,
                 sourceFileId=source_file_id,
                 fileName=new_file_name,
-                **kwargs
+                **kwargs,
             )
         except AccessDenied:
             raise SSECKeyError()
@@ -1142,16 +1192,22 @@ class B2RawHTTPApi(AbstractRawApi):
             kwargs['range'] = range_dict['Range']
         if destination_server_side_encryption is not None:
             assert destination_server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
+            )
+            kwargs['destinationServerSideEncryption'] = (
+                destination_server_side_encryption.serialize_to_json_for_request()
             )
-            kwargs['destinationServerSideEncryption'
-                  ] = destination_server_side_encryption.serialize_to_json_for_request()
         if source_server_side_encryption is not None:
             assert source_server_side_encryption.mode in (
-                EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                EncryptionMode.NONE,
+                EncryptionMode.SSE_B2,
+                EncryptionMode.SSE_C,
+            )
+            kwargs['sourceServerSideEncryption'] = (
+                source_server_side_encryption.serialize_to_json_for_request()
             )
-            kwargs['sourceServerSideEncryption'
-                  ] = source_server_side_encryption.serialize_to_json_for_request()
         try:
             return self._post_json(
                 api_url,
@@ -1160,7 +1216,7 @@ class B2RawHTTPApi(AbstractRawApi):
                 sourceFileId=source_file_id,
                 largeFileId=large_file_id,
                 partNumber=part_number,
-                **kwargs
+                **kwargs,
             )
         except AccessDenied:
             raise SSECKeyError()
@@ -1176,10 +1232,11 @@ class B2RawHTTPApi(AbstractRawApi):
                 'bucketId': bucket_id,
                 'eventNotificationRules': rules,
             },
-        )["eventNotificationRules"]
+        )['eventNotificationRules']
 
-    def get_bucket_notification_rules(self, api_url: str, account_auth_token: str,
-                                      bucket_id: str) -> list[NotificationRuleResponse]:
+    def get_bucket_notification_rules(
+        self, api_url: str, account_auth_token: str, bucket_id: str
+    ) -> list[NotificationRuleResponse]:
         return self._get_json(
             api_url,
             'b2_get_bucket_notification_rules',
@@ -1187,7 +1244,7 @@ class B2RawHTTPApi(AbstractRawApi):
             **{
                 'bucketId': bucket_id,
             },
-        )["eventNotificationRules"]
+        )['eventNotificationRules']
 
 
 def _add_range_header(headers, range_):
@@ -1195,4 +1252,4 @@ def _add_range_header(headers, range_):
         assert len(range_) == 2, range_
         assert (range_[0] + 0) <= (range_[1] + 0), range_  # not strings
         assert range_[0] >= 0, range_
-        headers['Range'] = "bytes=%d-%d" % range_
+        headers['Range'] = 'bytes=%d-%d' % range_
